#include "Status.h"
#include "Vram.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"
#include "VVram.h"

constexpr byte ScoreLeft = 25;

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + VramRowSize * 0 + ScoreLeft * VramStep, "SCORE");
    PrintS(Vram + VramRowSize * 1 + ScoreLeft * VramStep, "STAGE");
    PrintByteNumber2(Vram + VramRowSize * 1 + (ScoreLeft + 9) * VramStep, CurrentStage + 1);
    PrintS(Vram + VramRowSize * 2 + ScoreLeft * VramStep, "TIME");
    {
        static const byte[] RemainChars = {
            Char_MyFort + 2, 
            Char_MyFort + 3,
            Char_MyFort + 6,
            Char_MyFort + 7,
        };
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 5 + ScoreLeft * VramStep;
        if (RemainCount > 1) {
            ptr<byte> pChars;
            pChars = RemainChars;
            repeat (2) {
                pVram = PutU(pVram, *pChars);
                ++pChars;
            }
            pVram += VramRowSize - 2 * VramStep;
            repeat (2) {
                pVram = PutL(pVram, *pChars);
                ++pChars;
            }
            pVram += VramRowSize - 2 * VramStep;
            if (RemainCount > 2) {
                pVram = PrintC(pVram, RemainCount + (0x30 - 1));
            }
        }
    }
    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize * 0 + (ScoreLeft + 5) * VramStep, Score);
    PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(Vram + VramRowSize * 2 + (ScoreLeft + 8) * VramStep, StageTime);
}

void PrintGameOver()
{
    PrintS(Vram + VramRowSize * 3 + (VVramWidth - 9) / 2 * VramStep, "GAME OVER");
    PresendVram();
}

void PrintTimeUp() 
{
    PrintS(Vram + VramRowSize * 3 + (VVramWidth - 7) / 2 * VramStep, "TIME UP");
    PresendVram();
}

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    PrintStatus(); 
    {
        static const byte[] TitleBytes = {
            0x0f, 0x05, 0x0b, 0x00, 
            0x0f, 0x0a, 0x07, 0x0c, 
            0x0f, 0x00, 0x0f, 0x0c, 
            0x05, 0x05, 0x01, 0x04, 
            0x0e, 0x0b, 0x00, 0x05, 
            0x03, 0x0c, 0x03, 0x00, 
            0x07, 0x0d, 0x03, 0x00, 
            0x01, 0x04, 0x01, 0x00, 
            0x0f, 0x05, 0x04, 0x0d, 
            0x0f, 0x00, 0x00, 0x0c, 
            0x0f, 0x00, 0x00, 0x0c, 
            0x05, 0x00, 0x00, 0x04, 
            0x07, 0x01, 0x0f, 0x00, 
            0x03, 0x00, 0x0f, 0x00, 
            0x03, 0x00, 0x0f, 0x00, 
            0x01, 0x00, 0x05, 0x05, 
            0x00, 0x08, 0x07, 0x0d, 
            0x00, 0x0c, 0x03, 0x0c, 
            0x00, 0x0c, 0x03, 0x0c, 
            0x05, 0x00, 0x05, 0x05, 
            0x02, 0x05, 0x0f, 0x05, 
            0x03, 0x00, 0x0f, 0x00, 
            0x03, 0x00, 0x0f, 0x00, 
            0x00, 0x00, 0x05, 0x00, 
        };
        constexpr byte LogoLength = 6;
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 0 + (VVramWidth - LogoLength * 4) * VramStep / 2;
        p = TitleBytes;
        repeat (LogoLength) {
            repeat (2) {
                repeat (4) {
                    pVram = PutU(pVram, *p);
                    ++p;
                }
                pVram -= 4 * VramStep;
                repeat (4) {
                    pVram = PutL(pVram, *p);
                    ++p;
                }
                pVram += VramRowSize - 4 * VramStep;
            }
            pVram += 4 * VramStep - VramRowSize * 2;
        }
        PrintS(Vram + VramRowSize * 2 + (VVramWidth / 2 + LogoLength * 2 - 4) * VramStep, "MINI");
    }
    PrintS(Vram + VramRowSize * 3 + 5 * VramStep, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 4 + 5 * VramStep, "OR KANA KEY");
    PrintS(Vram + VramRowSize * 5 + 12 * VramStep, "INUFUTO 2022");
    PresendVram();
}