include "Vram.inc"
include "VVram.inc"

ext AsciiPattern, CharPattern
ext VVramFront_
ext YOffset_

LCDSTAT equ 40h
LCDDATA equ 41h


dseg
Vram_: public Vram_
    defs VramRowSize*VramHeight
Backup:
    defs VVramWidth*VramHeight*2


cseg
WaitLCD:
    push af
        do
        in a,(LCDSTAT)
            rlca
        while c | wend
    pop af
ret


; void ClearScreen();
cseg
Clear:
    do
        ld (hl),0 | inc hl
        dec bc
        ld a,c
        or b
    while nz | wend
ret
ClearScreen_: public ClearScreen_
    push af | push hl | push bc
        ld c,0
        do
            call WaitLCD
            xor a | out (LCDSTAT),a
            call WaitLCD
            ld a,10h | out (LCDSTAT),a
            call WaitLCD
            ld a,c
            or 0b0h | out (LCDSTAT),a
            xor a
            ld b,VramRowSize
            do
                call WaitLCD
                out (LCDDATA),a
            dwnz
            inc c
            ld a,c
            cp VramHeight
        while nz | wend

        ld hl,Vram_
        ld bc,VramRowSize*VramHeight
        call Clear

        ld hl,Backup
        ld bc,VVramWidth*VVramHeight
        call Clear
    pop bc | pop hl | pop af
ret


; void PresendVram();
cseg
PresendVram_: public PresendVram_
    push af | push hl | push bc
        ld hl,Vram_
        ld c,0
        do
            call WaitLCD
            xor a | out (LCDSTAT),a
            call WaitLCD
            ld a,10h | out (LCDSTAT),a
            call WaitLCD
            ld a,c
            or 0b0h | out (LCDSTAT),a
            xor a
            ld b,VramRowSize
            do
                call WaitLCD
                ld a,(hl) | inc hl
                out (LCDDATA),a
            dwnz
            inc c
            ld a,c
            cp VramHeight
        while nz | wend
    pop bc | pop hl | pop af
ret


; ptr<byte> PrintC(ptr<byte> pVram, byte c);
cseg
PrintC_: public PrintC_
    push af | push de | push bc
        ld a,e
        cp ' '
        if z
            xor a
        else
            cp '-'
            if z
                ld a,$01
            else
                cp 'A'
                if nc
                    add a, $0c-'A'
                else
                    add a, $02-'0'
                endif
            endif
        endif
        add a,a ;*2
        add a,a ;*4
        push hl
            ld l,a
            ld h,0
            ld de,AsciiPattern
            add hl,de
            ex de,hl
        pop hl
        ld b,4
        do
            ld a,(de) | inc de
            ld (hl),a | inc hl
            dec b
        while nz | wend
    pop bc | pop de | pop af
ret


CharPatternEDE:
    ex de,hl
        ld h,0
        add hl,hl
        ld bc,CharPattern
        add hl,bc
    ex de,hl
ret

; ptr<byte> PutU(ptr<byte> pVram, byte c);
cseg
PutU_: public PutU_
    push af | push de | push bc
        call CharPatternEDE
        ld b,2
        do
            ld a,(hl) | and 0f0h | ld c,a
            ld a,(de) | inc de
            push af
                and $0f
                or c
                ld (hl),a | inc hl

                ld a,(hl) | and 0f0h | ld c,a
            pop af
            rrca | rrca | rrca | rrca
            and $0f
            or c
            ld (hl),a | inc hl
        dwnz
    pop bc | pop de | pop af
ret

cseg
PutL_: public PutL_
    push af | push de | push bc
        call CharPatternEDE
        ld b,2
        do
            ld a,(hl) | and $0f | ld c,a
            ld a,(de) | inc de
            push af
                rrca | rrca | rrca | rrca
                and 0f0h
                or c
                ld (hl),a | inc hl

                ld a,(hl) | and $0f | ld c,a
            pop af
            and 0f0h
            or c
            ld (hl),a | inc hl
        dwnz
    pop bc | pop de | pop af
ret


; void VVramToVram();
cseg
VVramToVram_: public VVramToVram_
    push af | push hl | push de | push bc
        ld hl,VVramFront_
        ld de,VVramWidth
        ld a,(YOffset_)
        or a
        do | while nz
            add hl,de
            dec a
        wend
        ex de,hl

        ld hl,Backup
        exx
            ld hl,Vram_
        exx
        ld c,VramHeight
        do
            ld b,VVramWidth
            do
                ld a,(de) | inc de
                cp (hl)
                if nz
                    ld (hl),a
                    exx
                        ld e,a
                        call PutU_
                    exx
                else
                    exx
                        inc hl | inc hl | inc hl | inc hl
                    exx
                endif
                inc hl
            dwnz

            exx
                ld de,-VVramWidth*VramStep
                add hl,de
            exx

            ld b,VVramWidth
            do
                ld a,(de) | inc de
                cp (hl)
                if nz
                    ld (hl),a
                    exx
                        ld e,a
                        call PutL_
                    exx
                else
                    exx
                        inc hl | inc hl | inc hl | inc hl
                    exx
                endif
                inc hl
            dwnz

            exx
                ld de,VramRowSize-VVramWidth*VramStep
                add hl,de
            exx

            dec c
        while nz | wend
    pop bc | pop de | pop hl | pop af
ret
